{-----------------------------------------------------------------------------
   Unit Name: VariForm
   Purpose  : Demonstrate colors
   Author   : Vesa Lappalainen/Minja Manninen
   Date     : 01.11.2001
   Changed  : 13.11.2001/vl
   Todo     :

   Usage    : Choose colors you like and then press Ctrl+number
              to save them.  To pick saved color combonation
              press number (0-9)

  kelt vaal.kelt  pun
  musta  oranssi  pun
  musta  viol     vihr
-----------------------------------------------------------------------------}
unit VariForm;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ExtCtrls, StdCtrls, ActnList;

type
  TFormColors = class(TForm)
    PanelLeft: TPanel;
    PanelRight: TPanel;
    PanelLeftMid: TPanel;
    PanelRightMid: TPanel;
    Timer1: TTimer;
    ColorDialog1: TColorDialog;
    PanelButtons: TPanel;
    ButtonVasen: TButton;
    ButtonKeskusta: TButton;
    ButtonOikea: TButton;
    CBVilkku: TCheckBox;
    PanelVilkku: TPanel;
    ActionList1: TActionList;
    procedure Timer1Timer(Sender: TObject);
    procedure ButtonKeskustaClick(Sender: TObject);
    procedure CBVilkkuClick(Sender: TObject);
    procedure ButtonVasenClick(Sender: TObject);
    procedure ButtonOikeaClick(Sender: TObject);
    procedure PanelLeftClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormResize(Sender: TObject);
  private
    { Private declarations }
    procedure VaihdaVari;
    procedure VaihdaVarit(Panel1, Panel2: TPanel);
    function KysyVari(Panel: TPanel): boolean;
    procedure ReadColor(Sender: TObject);
    procedure SaveColor(Sender: TObject);
  public
    { Public declarations }
  end;

var
  FormColors: TFormColors;

implementation
uses IniFiles,IniName,kdouble;

{$R *.dfm}

const ReSizeFac = 7;
const ReSizeDiv = 3;

procedure TFormColors.VaihdaVarit(Panel1,Panel2:TPanel);
var temp : TColor;
begin
  temp := Panel1.Color;
  Panel1.Color := Panel2.Color;
  Panel2.Color := temp;
end;

procedure TFormColors.VaihdaVari;
begin
  VaihdaVarit(PanelLeft,PanelRight);
  VaihdaVarit(PanelLeftMid,PanelRightMid);
end;

procedure TFormColors.Timer1Timer(Sender: TObject);
begin
  VaihdaVari;
end;

function TFormColors.KysyVari(Panel:TPanel):boolean;
begin
  Result := false;
  ColorDialog1.Color := Panel.Color;
  if ( not ColorDialog1.Execute ) then exit;
  Panel.Color := ColorDialog1.Color;
  Result := true;
end;

procedure TFormColors.ButtonKeskustaClick(Sender: TObject);
begin
  if ( not KysyVari(PanelLeftMid) ) then exit;
  PanelRightMid.Color := PanelLeftMid.Color;
end;

procedure TFormColors.CBVilkkuClick(Sender: TObject);
begin
  Timer1.Enabled := CBVilkku.Checked;
end;

procedure TFormColors.ButtonVasenClick(Sender: TObject);
begin
  KysyVari(PanelLeft);
end;

procedure TFormColors.ButtonOikeaClick(Sender: TObject);
begin
  KysyVari(PanelRight);
end;

procedure TFormColors.PanelLeftClick(Sender: TObject);
begin
  if not ( sender is TPanel ) then exit;
  KysyVari(TPanel(Sender));
end;

procedure TFormColors.ReadColor(Sender: TObject);
var i:integer; s:string;
begin
  i := (Sender as TAction).Tag;
  s := IniReadString(TIniFile.Create(GetIniName),'SavedColors',IntToStr(i),'',true);
  PanelLeft.Color     := ExtractInt(s,',',0);
  PanelRight.Color    := ExtractInt(s,',',0);
  PanelLeftMid.Color  := ExtractInt(s,',',0);
  PanelRightMid.Color := ExtractInt(s,',',0);
end;

procedure TFormColors.SaveColor(Sender: TObject);
var i:integer; s:string;
begin
  i := (Sender as TAction).Tag;
  s := format('$%x,$%x,$%x,$%x',
     [PanelLeft.Color,PanelRight.Color,PanelLeftMid.Color,PanelRightMid.Color]);
  IniWriteString(TIniFile.Create(GetIniName),'SavedColors',IntToStr(i),s,true);
end;

procedure TFormColors.FormCreate(Sender: TObject);
var i:integer; a : TAction;
begin
  for i:=0 to 9 do begin
    a := TAction.Create(ActionList1);  a.ActionList := ActionList1;
    a.Caption := IntToStr(i);
    a.SecondaryShortCuts.Add('Ctrl+'+IntToStr(i));
    a.OnExecute := SaveColor;
    a.Tag := i;
    a := TAction.Create(ActionList1);  a.ActionList := ActionList1;
    a.Caption := IntToStr(i);
    a.SecondaryShortCuts.Add(IntToStr(i));
    a.OnExecute := ReadColor;
    a.Tag := i;
  end;
end;


procedure TFormColors.FormResize(Sender: TObject);
  procedure CountSize(Panel:TPanel;w,h:integer);
  var sx,sy : integer;
  begin
    sx := w * ReSizeDiv div ReSizeFac;
    sy := h * ReSizeDiv div ReSizeFac;
    Panel.SetBounds( (w-sx) div 2, (h-sy) div 2, sx,sy);
  end;
  procedure SetButton(b:TButton;x:integer);
  begin
    b.Left := x - b.Width div 2;
  end;
begin
  PanelLeft.Width := ClientWidth div 2;
  CountSize(PanelLeftMid,PanelLeft.Width,PanelLeft.Height);
  CountSize(PanelRightMid,PanelLeft.Width,PanelLeft.Height);
  SetButton(ButtonVasen,ClientWidth div 4);
  SetButton(ButtonKeskusta,ClientWidth div 2);
  SetButton(ButtonOikea,3*ClientWidth div 4);
end;

end.
